/**
 * @popperjs/core v2.0.0-next.12
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.Popper = {}));
}(this, (function (exports) { 'use strict';

  var getBoundingClientRect = (function (element) {
    var rect = JSON.parse(JSON.stringify(element.getBoundingClientRect()));
    rect.x = rect.left;
    rect.y = rect.top;
    return rect;
  });

  var getNodeName = (function (element) {
    return element ? (element.nodeName || '').toUpperCase() : null;
  });

  var getParentNode = (function (element) {
    if (getNodeName(element) === 'HTML') {
      // DocumentElement detectedF
      return element;
    }

    return element.parentNode || // DOM Element detected
    // $FlowFixMe: need a better way to handle this...
    element.host || // ShadowRoot detected
    document.ownerDocument || // Fallback to ownerDocument if available
    document.documentElement // Or to documentElement if everything else fails
    ;
  });

  function getWindow(node) {
    var ownerDocument = node != null && typeof node === 'object' && node.hasOwnProperty('ownerDocument') ? // $FlowFixMe: assume ownerDocument to be the one we are looking for
    node.ownerDocument : null;
    return ownerDocument != null ? ownerDocument.defaultView : window;
  }

  function getComputedStyle(element) {
    return getWindow(element).getComputedStyle(element);
  }

  function isElement(node) {
    var OwnElement = getWindow(node).Element;
    return node instanceof OwnElement;
  }

  function isHTMLElement(node) {
    var OwnElement = getWindow(node).HTMLElement;
    return node instanceof OwnElement;
  }

  function getScrollParent(node) {
    if (['HTML', 'BODY', '#DOCUMENT'].includes(getNodeName(node))) {
      return node.ownerDocument.body;
    }

    if (isHTMLElement(node)) {
      // Firefox want us to check `-x` and `-y` variations as well
      var _getComputedStyle = getComputedStyle(node),
          overflow = _getComputedStyle.overflow,
          overflowX = _getComputedStyle.overflowX,
          overflowY = _getComputedStyle.overflowY;

      if (/(auto|scroll|overlay)/.test(overflow + overflowY + overflowX)) {
        return node;
      }
    }

    return getScrollParent(getParentNode(node));
  }

  function listScrollParents(element, list) {
    if (list === void 0) {
      list = [];
    }

    var scrollParent = getScrollParent(element);
    var isBody = getNodeName(scrollParent) === 'BODY';
    var target = isBody ? getWindow(scrollParent) : scrollParent;
    var updatedList = list.concat(target);
    return isBody ? updatedList : updatedList.concat(listScrollParents(getParentNode(target)));
  }

  function getWindowScroll(node) {
    var win = getWindow(node);
    var scrollLeft = win.pageXOffset;
    var scrollTop = win.pageYOffset;
    return {
      scrollLeft: scrollLeft,
      scrollTop: scrollTop
    };
  }

  function getHTMLElementScroll(element) {
    return {
      scrollLeft: element.scrollLeft,
      scrollTop: element.scrollTop
    };
  }

  function getElementScroll(node) {
    if (node === getWindow(node) || !isHTMLElement(node)) {
      return getWindowScroll(node);
    } else {
      return getHTMLElementScroll(node);
    }
  }

  var getScrollSum = (function (scrollParents) {
    return scrollParents.reduce(function (scroll, scrollParent) {
      var nodeScroll = getElementScroll(scrollParent);
      scroll.scrollTop += nodeScroll.scrollTop;
      scroll.scrollLeft += nodeScroll.scrollLeft;
      return scroll;
    }, {
      scrollTop: 0,
      scrollLeft: 0
    });
  });

  function getOffsetParent(element) {
    var offsetParent = isHTMLElement(element) ? element.offsetParent : null;
    var window = getWindow(element);

    if (getNodeName(offsetParent) === 'BODY') {
      return window;
    }

    return offsetParent || window;
  }

  var unwrapVirtualElement = (function (element) {
    return isElement(element) ? element : element.contextElement;
  });

  // offsetParent

  var getRectRelativeToOffsetParent = (function (element, isFixed) {
    if (isFixed === void 0) {
      isFixed = false;
    }

    var unwrappedElement = unwrapVirtualElement(element);
    var rect = getBoundingClientRect(element);
    var scrollParentsScrollSum = getScrollSum(listScrollParents(unwrappedElement));
    var directOffsetParent = getOffsetParent(unwrappedElement);
    var directOffsetParentRect = isElement(directOffsetParent) && !isFixed ? getBoundingClientRect(directOffsetParent) : {
      left: 0,
      top: 0
    };
    var ancestorOffsetParents = isFixed ? [directOffsetParent] : [];
    var currentOffsetParent = directOffsetParent;

    while (isElement(currentOffsetParent)) {
      currentOffsetParent = getOffsetParent(currentOffsetParent);
      ancestorOffsetParents.push(currentOffsetParent);
    }

    var ancestorOffsetParentScrollSum = getScrollSum(ancestorOffsetParents);
    var width = rect.width;
    var height = rect.height;
    var x = rect.left + scrollParentsScrollSum.scrollLeft - ancestorOffsetParentScrollSum.scrollLeft - directOffsetParentRect.left;
    var y = rect.top + scrollParentsScrollSum.scrollTop - ancestorOffsetParentScrollSum.scrollTop - directOffsetParentRect.top;
    return {
      width: width,
      height: height,
      x: x,
      y: y
    };
  });

  var getElementMargins = (function (element) {
    // get the element margins, we need them to properly align the popper
    var styles = getComputedStyle(element);
    var top = parseFloat(styles.marginTop) || 0;
    var right = parseFloat(styles.marginRight) || 0;
    var bottom = parseFloat(styles.marginBottom) || 0;
    var left = parseFloat(styles.marginLeft) || 0;
    return {
      top: top,
      right: right,
      bottom: bottom,
      left: left
    };
  });

  var addClientRectMargins = (function (rect, element) {
    var margins = getElementMargins(element);
    return {
      width: rect.width + margins.left + margins.right,
      height: rect.height + margins.top + margins.bottom,
      y: rect.y - margins.top,
      x: rect.x - margins.left
    };
  });

  var unwrapJqueryElement = (function (element) {
    return (// $FlowFixMe: need to get type refinement work
      element.hasOwnProperty('jquery') ? element[0] : element
    );
  });

  // source: https://stackoverflow.com/questions/49875255
  var order = function order(modifiers) {
    // indexed by name
    var mapped = modifiers.reduce(function (mem, i) {
      mem[i.name] = i;
      return mem;
    }, {}); // inherit all dependencies for a given name

    var inherited = function inherited(i) {
      return mapped[i] && mapped[i].requires ? mapped[i].requires.reduce(function (mem, i) {
        return [].concat(mem, [i], inherited(i));
      }, []) : [];
    }; // order ...


    var ordered = modifiers.sort(function (a, b) {
      return !!~inherited(b.name).indexOf(a.name) ? -1 : 1;
    });
    return ordered;
  };

  var orderModifiers = (function (modifiers) {
    return ['read', 'main', 'afterMain', 'write'].reduce(function (acc, currentPhase) {
      return acc.concat(order(modifiers.filter(function (modifier) {
        return modifier.phase === currentPhase;
      })));
    }, []);
  });

  function microtaskDebounce(fn) {
    var pending;
    return function () {
      if (!pending) {
        pending = new Promise(function (resolve) {
          Promise.resolve().then(function () {
            pending = undefined;
            resolve(fn());
          });
        });
      }

      return pending;
    };
  }

  var format = (function (str) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    return [].concat(args).reduce(function (p, c) {
      return p.replace(/%s/, c);
    }, str);
  });

  var top = 'top';
  var bottom = 'bottom';
  var right = 'right';
  var left = 'left';
  var basePlacements = [top, bottom, right, left];
  var start = 'start';
  var end = 'end';
  var placements = basePlacements.reduce(function (acc, placement) {
    return acc.concat([placement, placement + "-" + start, placement + "-" + end]);
  }, []); // modifiers that need to read the DOM

  var read = 'read'; // pure-logic modifiers

  var main = 'main'; // pure-logic modifiers that run after the main phase (such as computeStyles)

  var write = 'write';

  var ERROR_MESSAGE = 'Popper: modifier "%s" provided an invalid %s property, expected %s but got %s';
  var VALID_PROPERTIES = ['name', 'enabled', 'phase', 'fn', 'onLoad', 'requires', 'options'];
  var validateModifiers = (function (modifiers) {
    modifiers.forEach(function (modifier) {
      Object.keys(modifier).forEach(function (key) {
        switch (key) {
          case 'name':
            if (typeof modifier.name !== 'string') {
              console.error(format(ERROR_MESSAGE, String(modifier.name), '"name"', '"string"', "\"" + String(modifier.name) + "\""));
            }

            break;

          case 'enabled':
            if (typeof modifier.enabled !== 'boolean') {
              console.error(format(ERROR_MESSAGE, modifier.name, '"enabled"', '"boolean"', "\"" + String(modifier.enabled) + "\""));
            }

          case 'phase':
            if (![read, main, write].includes(modifier.phase)) {
              console.error(format(ERROR_MESSAGE, modifier.name, '"phase"', 'either "read", "main" or "write"', "\"" + String(modifier.phase) + "\""));
            }

            break;

          case 'fn':
            if (typeof modifier.fn !== 'function') {
              console.error(format(ERROR_MESSAGE, modifier.name, '"fn"', '"function"', "\"" + String(modifier.fn) + "\""));
            }

            break;

          case 'onLoad':
            if (typeof modifier.onLoad !== 'function') {
              console.error(format(ERROR_MESSAGE, modifier.name, '"onLoad"', '"function"', "\"" + String(modifier.fn) + "\""));
            }

            break;

          case 'onDestroy':
            if (typeof modifier.onLoad !== 'function') {
              console.error(format(ERROR_MESSAGE, modifier.name, '"onDestroy"', '"function"', "\"" + String(modifier.fn) + "\""));
            }

            break;

          case 'requires':
            if (!Array.isArray(modifier.requires)) {
              console.error(format(ERROR_MESSAGE, modifier.name, '"requires"', '"array"', "\"" + String(modifier.requires) + "\""));
            }

            break;

          case 'options':
          case 'data':
            break;

          default:
            console.error("PopperJS: an invalid property has been provided to the \"" + modifier.name + "\" modifier, valid properties are " + VALID_PROPERTIES.map(function (s) {
              return "\"" + s + "\"";
            }).join(', ') + "; but \"" + key + "\" was provided.");
        }
      });
    });
  });

  // DOM Utils
  var INVALID_ELEMENT_ERROR = 'Popper: Invalid reference or popper argument provided to Popper, they must be either a valid DOM element, virtual element, or a jQuery-wrapped DOM element.';
  var INFINITE_LOOP_ERROR = 'Popper: An infinite loop in the modifiers cycle has been detected! The cycle has been interrupted to prevent a browser crash.';

  var areValidElements = function areValidElements() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return !args.some(function (element) {
      return !(element && typeof element.getBoundingClientRect === 'function');
    });
  };

  var defaultOptionsValue = {
    placement: 'bottom',
    eventListeners: {
      scroll: true,
      resize: true
    },
    modifiers: [],
    strategy: 'absolute'
  };
  function popperGenerator(generatorOptions) {
    if (generatorOptions === void 0) {
      generatorOptions = {};
    }

    var _generatorOptions = generatorOptions,
        _generatorOptions$def = _generatorOptions.defaultModifiers,
        defaultModifiers = _generatorOptions$def === void 0 ? [] : _generatorOptions$def,
        _generatorOptions$def2 = _generatorOptions.defaultOptions,
        defaultOptions = _generatorOptions$def2 === void 0 ? defaultOptionsValue : _generatorOptions$def2;
    return function createPopper(reference, popper, options) {
      if (options === void 0) {
        options = defaultOptions;
      }

      // Unwrap `reference` and `popper` elements in case they are
      // wrapped by jQuery, otherwise consume them as is
      var referenceElement = unwrapJqueryElement(reference);
      var popperElement = unwrapJqueryElement(popper);
      var state = {
        placement: 'bottom',
        orderedModifiers: [],
        options: defaultOptions,
        modifiersData: {},
        elements: {
          reference: referenceElement,
          popper: popperElement
        }
      };
      var instance = {
        setOptions: function setOptions(options) {
          // Store options into state
          state.options = Object.assign({}, defaultOptions, {}, options);
          state.scrollParents = {
            reference: listScrollParents(referenceElement),
            popper: listScrollParents(popperElement)
          }; // Validate the provided modifiers so that the consumer will get warned
          // of one of the custom modifiers is invalid for any reason

          {
            validateModifiers(state.options.modifiers);
          } // Order `options.modifiers` so that the dependencies are fulfilled
          // once the modifiers are executed


          state.orderedModifiers = orderModifiers([].concat(defaultModifiers, state.options.modifiers)) // Apply user defined preferences to modifiers
          .map(function (modifier) {
            return Object.assign({}, modifier, {}, state.options.modifiers.find(function (_ref) {
              var name = _ref.name;
              return name === modifier.name;
            }));
          });
        },
        // Syncronous and forcefully executed update
        // it will always be executed even if not necessary, usually NOT needed
        // use Popper#update instead
        forceUpdate: function forceUpdate() {
          var _state$elements = state.elements,
              referenceElement = _state$elements.reference,
              popperElement = _state$elements.popper; // Don't proceed if `reference` or `popper` are not valid elements anymore

          if (!areValidElements(referenceElement, popperElement)) {
            {
              console.error(INVALID_ELEMENT_ERROR);
            }

            return;
          }

          var isFixed = state.options.strategy === 'fixed'; // Get initial measurements
          // these are going to be used to compute the initial popper offsets
          // and as cache for any modifier that needs them later

          state.measures = {
            reference: getRectRelativeToOffsetParent(referenceElement, isFixed),
            // CSS marginsc an be applied to popper elements to quickly
            // apply offsets dynamically based on some CSS selectors.
            // For this reason we include margins in this calculation.
            popper: addClientRectMargins(getRectRelativeToOffsetParent(popperElement, isFixed), popperElement)
          }; // Modifiers have the ability to read the current Popper state, included
          // the popper offsets, and modify it to address specifc cases

          state.reset = false; // Cache the placement in cache to make it available to the modifiers
          // modifiers will modify this one (rather than the one in options)

          state.placement = state.options.placement;
          state.orderedModifiers.forEach(function (modifier) {
            return state.modifiersData[modifier.name] = Object.assign({}, modifier.data);
          });
          var __debug_loops__ = 0;

          for (var index = 0; index < state.orderedModifiers.length; index++) {
            {
              __debug_loops__ += 1;

              if (__debug_loops__ > 100) {
                console.error(INFINITE_LOOP_ERROR);
                break;
              }
            }

            if (state.reset === true) {
              state.reset = false;
              index = -1;
              continue;
            }

            var _state$orderedModifie = state.orderedModifiers[index],
                fn = _state$orderedModifie.fn,
                enabled = _state$orderedModifie.enabled,
                _options = _state$orderedModifie.options,
                name = _state$orderedModifie.name;

            if (enabled && typeof fn === 'function') {
              state = fn({
                state: state,
                options: _options,
                name: name,
                instance: instance
              });
            }
          }
        },
        // Async and optimistically optimized update
        // it will not be executed if not necessary
        // debounced, so that it only runs at most once-per-tick
        update: microtaskDebounce(function () {
          return (// prettier-ignore
            new Promise(function (resolve) {
              instance.forceUpdate();
              resolve(state);
            })
          );
        }),
        destroy: function destroy() {
          // Run `onDestroy` modifier methods
          state.orderedModifiers.forEach(function (_ref2) {
            var onDestroy = _ref2.onDestroy,
                enabled = _ref2.enabled,
                name = _ref2.name;
            return enabled && onDestroy && onDestroy({
              state: state,
              name: name,
              instance: instance
            });
          });
        }
      }; // Don't proceed if `reference` or `popper` are invalid elements

      if (!areValidElements(referenceElement, popperElement)) {
        {
          console.error(INVALID_ELEMENT_ERROR);
        }

        return instance;
      }

      instance.setOptions(options); // Modifiers have the opportunity to execute some arbitrary code before
      // the first update cycle is ran, the order of execution will be the same
      // defined by the modifier dependencies directive.
      // The `onLoad` function may add or alter the options of themselves

      state.orderedModifiers.forEach(function (_ref3) {
        var onLoad = _ref3.onLoad,
            enabled = _ref3.enabled,
            name = _ref3.name;
        return enabled && onLoad && onLoad({
          state: state,
          name: name,
          instance: instance
        });
      });
      instance.update();
      return instance;
    };
  }
  var createPopper = popperGenerator();

  function onLoad(_ref) {
    var state = _ref.state,
        _ref$options = _ref.options,
        options = _ref$options === void 0 ? {} : _ref$options,
        instance = _ref.instance;
    var _options$scroll = options.scroll,
        scroll = _options$scroll === void 0 ? true : _options$scroll,
        _options$resize = options.resize,
        resize = _options$resize === void 0 ? true : _options$resize;

    if (scroll) {
      var scrollParents = [].concat(state.scrollParents.reference, state.scrollParents.popper);
      scrollParents.forEach(function (scrollParent) {
        return scrollParent.addEventListener('scroll', instance.update, {
          passive: true
        });
      });
    }

    if (resize) {
      var window = getWindow(state.elements.popper);
      window.addEventListener('resize', instance.update, {
        passive: true
      });
    }
  }
  function onDestroy(_ref2) {
    var state = _ref2.state,
        _ref2$options = _ref2.options,
        options = _ref2$options === void 0 ? {} : _ref2$options,
        instance = _ref2.instance;
    var _options$scroll2 = options.scroll,
        scroll = _options$scroll2 === void 0 ? true : _options$scroll2,
        _options$resize2 = options.resize,
        resize = _options$resize2 === void 0 ? true : _options$resize2;

    if (scroll) {
      // Remove scroll event listeners
      var scrollParents = [].concat(state.scrollParents.reference, state.scrollParents.popper);
      scrollParents.forEach(function (scrollParent) {
        return scrollParent.removeEventListener('scroll', instance.update);
      });
    }

    if (resize) {
      // Remove resize event listeners
      var window = getWindow(state.elements.popper);
      window.removeEventListener('resize', instance.update);
    }
  }
  var eventListeners = {
    name: 'offset',
    enabled: true,
    phase: 'write',
    fn: function fn(_ref3) {
      var state = _ref3.state;
      return state;
    },
    onLoad: onLoad,
    onDestroy: onDestroy
  };

  var getBasePlacement = (function (placement) {
    return placement.split('-')[0];
  });

  var getVariationPlacement = (function (placement) {
    return placement.split('-')[1];
  });

  var getMainAxisFromPlacement = (function (placement) {
    return ['top', 'bottom'].includes(placement) ? 'x' : 'y';
  });

  var computeOffsets = (function (_ref) {
    var reference = _ref.reference,
        element = _ref.element,
        placement = _ref.placement,
        scroll = _ref.scroll;
    var basePlacement = placement ? getBasePlacement(placement) : null;
    var variationPlacement = placement ? getVariationPlacement(placement) : null;
    var scrollTop = scroll.scrollTop,
        scrollLeft = scroll.scrollLeft;
    var offsets;

    switch (basePlacement) {
      case top:
        offsets = {
          x: reference.x + reference.width / 2 - element.width / 2 - scrollLeft,
          y: reference.y - element.height - scrollTop
        };
        break;

      case bottom:
        offsets = {
          x: reference.x + reference.width / 2 - element.width / 2 - scrollLeft,
          y: reference.y + reference.height - scrollTop
        };
        break;

      case right:
        offsets = {
          x: reference.x + reference.width - scrollLeft,
          y: reference.y + reference.height / 2 - element.height / 2 - scrollTop
        };
        break;

      case left:
        offsets = {
          x: reference.x - element.width - scrollLeft,
          y: reference.y + reference.height / 2 - element.height / 2 - scrollTop
        };
        break;

      default:
        offsets = {
          x: reference.x - scrollLeft,
          y: reference.y - scrollTop
        };
    }

    var mainAxis = basePlacement ? getMainAxisFromPlacement(basePlacement) : null;

    if (mainAxis != null) {
      var len = mainAxis === 'y' ? 'height' : 'width';

      switch (variationPlacement) {
        case start:
          offsets[mainAxis] = Math.floor(offsets[mainAxis]) - Math.floor(reference[len] / 2 - element[len] / 2);
          break;

        case end:
          offsets[mainAxis] = Math.floor(offsets[mainAxis]) + Math.ceil(reference[len] / 2 - element[len] / 2);
          break;
      }
    }

    return offsets;
  });

  function getCommonTotalScroll(referenceElement, referenceScrollParents, popperScrollParents, limiter) {
    var offsetParent = getOffsetParent(referenceElement);
    var index = referenceScrollParents.findIndex(function (node) {
      return node === (limiter || offsetParent);
    });
    var scrollParents = referenceScrollParents.slice(0, index === -1 ? undefined : index);
    return getScrollSum(scrollParents);
  }

  function popperOffsets(_ref) {
    var state = _ref.state;
    // Offsets are the actual position the popper needs to have to be
    // properly positioned near its reference element
    // This is the most basic placement, and will be adjusted by
    // the modifiers in the next step
    state.modifiersData.popperOffsets = computeOffsets({
      reference: state.measures.reference,
      element: state.measures.popper,
      strategy: 'absolute',
      placement: state.placement,
      scroll: getCommonTotalScroll(unwrapVirtualElement(state.elements.reference), state.scrollParents.reference, state.scrollParents.popper)
    });
    return state;
  }
  var popperOffsets$1 = {
    name: 'popperOffsets',
    enabled: true,
    phase: 'read',
    fn: popperOffsets,
    data: {}
  };

  // This modifier takes the Popper state and prepares some StyleSheet properties
  // that can be applied to the popper element to make it render in the expected position.
  var mapToStyles = function mapToStyles(_ref) {
    var offsets = _ref.offsets,
        position = _ref.position,
        gpuAcceleration = _ref.gpuAcceleration;

    // @1x displays in Chrome can be blurry due to non-rounded offsets, but this
    // introduces slight positioning errors. TODO: somehow solve this better
    // by default it is active, disable it only if explicitly set to false
    if (gpuAcceleration === false || window.devicePixelRatio < 2) {
      return {
        top: offsets.y + "px",
        left: offsets.x + "px",
        position: position
      };
    } else {
      return {
        top: '0px',
        left: '0px',
        transform: "translate3d(" + offsets.x + "px, " + offsets.y + "px, 0)",
        position: position
      };
    }
  };
  function computeStyles(_ref2) {
    var state = _ref2.state,
        _ref2$options = _ref2.options,
        options = _ref2$options === void 0 ? {} : _ref2$options;
    var _options$gpuAccelerat = options.gpuAcceleration,
        gpuAcceleration = _options$gpuAccelerat === void 0 ? true : _options$gpuAccelerat;
    state.modifiersData.computeStyles = {
      styles: {
        // popper offsets are always available
        popper: mapToStyles({
          offsets: state.modifiersData.popperOffsets,
          position: state.options.strategy,
          gpuAcceleration: gpuAcceleration
        }),
        // arrow offsets may not be available
        arrow: state.modifiersData.arrow != null ? mapToStyles({
          offsets: state.modifiersData.arrow,
          position: 'absolute',
          gpuAcceleration: false
        }) : undefined
      },
      attributes: {
        popper: {
          'data-popper-placement': state.placement
        }
      }
    };
    return state;
  }
  var computeStyles$1 = {
    name: 'computeStyles',
    enabled: true,
    phase: 'afterMain',
    fn: computeStyles,
    data: {}
  };

  // and applies them to the HTMLElements such as popper and arrow

  function applyStyles(_ref) {
    var state = _ref.state;
    Object.keys(state.elements).forEach(function (name) {
      var data = state.modifiersData.computeStyles;
      var style = data.styles[name] || {};
      var attributes = data.attributes[name] || {};
      var element = state.elements[name]; // arrow is optional + virtual elements

      if (!isHTMLElement(element) || !getNodeName(element)) {
        return;
      }

      Object.assign(element.style, style);
      Object.entries(attributes).forEach(function (args) {
        return element.setAttribute.apply(element, args);
      });
    });
    return state;
  }
  function onDestroy$1(_ref2) {
    var state = _ref2.state;
    var data = state.modifiersData.computeStyles;
    Object.keys(state.elements).forEach(function (name) {
      var element = state.elements[name];
      var styleProperties = Object.keys(data.styles.hasOwnProperty(name) ? Object.assign({}, data.styles[name]) : {});
      var attributes = data.attributes[name] || {}; // Set all values to an empty string to unset them

      var style = styleProperties.reduce(function (style, property) {
        var _Object$assign;

        return Object.assign({}, style, (_Object$assign = {}, _Object$assign[String(property)] = '', _Object$assign));
      }, {}); // arrow is optional + virtual elements

      if (!isHTMLElement(element) || !getNodeName(element)) {
        return;
      } // Flow doesn't support to extend this property, but it's the most
      // effective way to apply styles to an HTMLElement
      // $FlowIgnore


      Object.assign(element.style, style);
      Object.keys(attributes).forEach(function (attribute) {
        return element.removeAttribute(attribute);
      });
    });
  }
  var applyStyles$1 = {
    name: 'applyStyles',
    enabled: true,
    phase: 'write',
    fn: applyStyles,
    onDestroy: onDestroy$1,
    requires: ['computeStyles']
  };

  var createPopper$1 = popperGenerator({
    defaultModifiers: [eventListeners, popperOffsets$1, computeStyles$1, applyStyles$1]
  });

  exports.createPopper = createPopper$1;
  exports.popperGenerator = popperGenerator;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=popper-lite.js.map
