import getBoundingClientRect from './getBoundingClientRect';
import listScrollParents from './listScrollParents';
import getScrollSum from './getScrollSum';
import getOffsetParent from './getOffsetParent';
import unwrapVirtualElement from './unwrapVirtualElement';
import { isElement } from './instanceOf'; // Returns the width, height and offsets of the provided element relative to the
// offsetParent

export default (function (element, isFixed) {
  if (isFixed === void 0) {
    isFixed = false;
  }

  var unwrappedElement = unwrapVirtualElement(element);
  var rect = getBoundingClientRect(element);
  var scrollParentsScrollSum = getScrollSum(listScrollParents(unwrappedElement));
  var directOffsetParent = getOffsetParent(unwrappedElement);
  var directOffsetParentRect = isElement(directOffsetParent) && !isFixed ? getBoundingClientRect(directOffsetParent) : {
    left: 0,
    top: 0
  };
  var ancestorOffsetParents = isFixed ? [directOffsetParent] : [];
  var currentOffsetParent = directOffsetParent;

  while (isElement(currentOffsetParent)) {
    currentOffsetParent = getOffsetParent(currentOffsetParent);
    ancestorOffsetParents.push(currentOffsetParent);
  }

  var ancestorOffsetParentScrollSum = getScrollSum(ancestorOffsetParents);
  var width = rect.width;
  var height = rect.height;
  var x = rect.left + scrollParentsScrollSum.scrollLeft - ancestorOffsetParentScrollSum.scrollLeft - directOffsetParentRect.left;
  var y = rect.top + scrollParentsScrollSum.scrollTop - ancestorOffsetParentScrollSum.scrollTop - directOffsetParentRect.top;
  return {
    width: width,
    height: height,
    x: x,
    y: y
  };
});